<?php

namespace WPIS;

require_once __DIR__ . '/Shortcodes/VueEstateListingShortcode.php';
require_once __DIR__ . '/Shortcodes/EstateDetailsListShortcode.php';
require_once __DIR__ . '/Shortcodes/EstateMetaLabelShortcode.php';
require_once __DIR__ . '/Shortcodes/EstateMetaValueShortcode.php';
require_once __DIR__ . '/Shortcodes/EstateMetaGroupShortcode.php';
require_once __DIR__ . '/Shortcodes/FormContactShortcode.php';
require_once __DIR__ . '/Shortcodes/FormEvaluationShortcode.php';
require_once __DIR__ . '/Shortcodes/FormEstateShortcode.php';

class DatabaseManager {

    static public function install() {
        self::createContactTable();
        self::createConnectionTable();
        self::createSettingsTable();
    }

    static public function uninstall() {
        self::removeTable(WPIS::WP_TABLE_CONNECTION);
        self::removeTable(WPIS::WP_TABLE_SETTINGS);
        self::removeTable(WPIS::WP_TABLE_CONTACTS);
    }

    static public function createContactTable() {
        global $wpdb;

        $charset = $wpdb->get_charset_collate();
        $table = $wpdb->prefix . WPIS::WP_TABLE_CONTACTS;

        $query = "CREATE TABLE IF NOT EXISTS $table (
          id INT AUTO_INCREMENT PRIMARY KEY,
          source tinytext NOT NULL,
          firstname tinytext NOT NULL,
          lastname tinytext NOT NULL,
          email tinytext NOT NULL,
          form_data TEXT NOT NULL,
          email_recipient BOOLEAN NOT NULL,
          email_notification BOOLEAN NOT NULL,
          software_contact_id tinytext NULL,
          created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP
        ) $charset;";

        $wpdb->query( $query );
    }

    static public function createConnectionTable() {
        global $wpdb;

        $charset = $wpdb->get_charset_collate();
        $table = $wpdb->prefix . WPIS::WP_TABLE_CONNECTION;

        $query = "CREATE TABLE IF NOT EXISTS $table (
            access_id tinytext NOT NULL,
            name tinytext NOT NULL,
            account tinytext NOT NULL,
            token TEXT NOT NULL,
            activated_at TIMESTAMP NOT NULL,
            created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP
          ) $charset;";

        $wpdb->query( $query );
    }

    static public function createSettingsTable() {
        global $wpdb;

        $charset = $wpdb->get_charset_collate();
        $table = $wpdb->prefix . WPIS::WP_TABLE_SETTINGS;

        $query = "CREATE TABLE IF NOT EXISTS $table (
            name VARCHAR(50) UNIQUE NOT NULL,
            value VARCHAR(50),
            created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP
          ) $charset;";

        $wpdb->query( $query );
    }

    static private function removeTable($table){
        global $wpdb;
        $query = 'DROP TABLE IF EXISTS ' . $wpdb->prefix . $table . ';';
        $wpdb->query($query);
    }

}